@@ -1,27 +1,27 @@
-use std::fmt::Debug;
+use core::fmt::Debug;
 
 use crossbeam_channel as channel;
 use serde::{Serialize, Serializer};
 
-use ibc::ics02_client::client_consensus::{AnyConsensusState, AnyConsensusStateWithHeight};
-use ibc::ics02_client::client_state::{AnyClientState, IdentifiedAnyClientState};
-use ibc::ics02_client::events::UpdateClient;
-use ibc::ics02_client::misbehaviour::MisbehaviourEvidence;
-use ibc::ics03_connection::connection::IdentifiedConnectionEnd;
-use ibc::ics04_channel::channel::IdentifiedChannelEnd;
-use ibc::ics04_channel::packet::{PacketMsgType, Sequence};
-use ibc::query::QueryTxRequest;
 use ibc::{
+    core::{
+        ics02_client::client_consensus::{AnyConsensusState, AnyConsensusStateWithHeight},
+        ics02_client::client_state::{AnyClientState, IdentifiedAnyClientState},
+        ics02_client::events::UpdateClient,
+        ics02_client::header::AnyHeader,
+        ics02_client::misbehaviour::MisbehaviourEvidence,
+        ics03_connection::connection::{ConnectionEnd, IdentifiedConnectionEnd},
+        ics03_connection::version::Version,
+        ics04_channel::channel::{ChannelEnd, IdentifiedChannelEnd},
+        ics04_channel::packet::{PacketMsgType, Sequence},
+        ics23_commitment::commitment::CommitmentPrefix,
+        ics24_host::identifier::ChainId,
+        ics24_host::identifier::ChannelId,
+        ics24_host::identifier::{ClientId, ConnectionId, PortId},
+    },
     events::IbcEvent,
-    ics02_client::header::AnyHeader,
-    ics03_connection::connection::ConnectionEnd,
-    ics03_connection::version::Version,
-    ics04_channel::channel::ChannelEnd,
-    ics23_commitment::commitment::CommitmentPrefix,
-    ics24_host::identifier::ChainId,
-    ics24_host::identifier::ChannelId,
-    ics24_host::identifier::{ClientId, ConnectionId, PortId},
     proofs::Proofs,
+    query::{QueryBlockRequest, QueryTxRequest},
     signer::Signer,
     Height,
 };
@@ -36,9 +36,12 @@ use ibc_proto::ibc::core::commitment::v1::MerkleProof;
 use ibc_proto::ibc::core::connection::v1::QueryClientConnectionsRequest;
 use ibc_proto::ibc::core::connection::v1::QueryConnectionsRequest;
 
-use crate::{connection::ConnectionMsgType, error::Error, keyring::KeyEntry};
+use crate::{
+    chain::StatusResponse, config::ChainConfig, connection::ConnectionMsgType, error::Error,
+    keyring::KeyEntry,
+};
 
-use super::{reply_channel, ChainHandle, ChainRequest, ReplyTo, Subscription};
+use super::{reply_channel, ChainHandle, ChainRequest, HealthCheck, ReplyTo, Subscription};
 
 #[derive(Debug, Clone)]
 pub struct ProdChainHandle {
@@ -80,6 +83,10 @@ impl ChainHandle for ProdChainHandle {
         self.chain_id.clone()
     }
 
+    fn health_check(&self) -> Result<HealthCheck, Error> {
+        self.send(|reply_to| ChainRequest::HealthCheck { reply_to })
+    }
+
     fn shutdown(&self) -> Result<(), Error> {
         self.send(|reply_to| ChainRequest::Shutdown { reply_to })
     }
@@ -112,6 +119,10 @@ impl ChainHandle for ProdChainHandle {
         self.send(|reply_to| ChainRequest::Signer { reply_to })
     }
 
+    fn config(&self) -> Result<ChainConfig, Error> {
+        self.send(|reply_to| ChainRequest::Config { reply_to })
+    }
+
     fn get_key(&self) -> Result<KeyEntry, Error> {
         self.send(|reply_to| ChainRequest::Key { reply_to })
     }
@@ -123,8 +134,8 @@ impl ChainHandle for ProdChainHandle {
         })
     }
 
-    fn query_latest_height(&self) -> Result<Height, Error> {
-        self.send(|reply_to| ChainRequest::QueryLatestHeight { reply_to })
+    fn query_status(&self) -> Result<StatusResponse, Error> {
+        self.send(|reply_to| ChainRequest::QueryStatus { reply_to })
     }
 
     fn query_clients(
@@ -309,8 +320,16 @@ impl ChainHandle for ProdChainHandle {
         })
     }
 
-    fn build_client_state(&self, height: Height) -> Result<AnyClientState, Error> {
-        self.send(|reply_to| ChainRequest::BuildClientState { height, reply_to })
+    fn build_client_state(
+        &self,
+        height: Height,
+        dst_config: ChainConfig,
+    ) -> Result<AnyClientState, Error> {
+        self.send(|reply_to| ChainRequest::BuildClientState {
+            height,
+            dst_config,
+            reply_to,
+        })
     }
 
     fn build_consensus_state(
@@ -418,7 +437,14 @@ impl ChainHandle for ProdChainHandle {
     }
 
     fn query_txs(&self, request: QueryTxRequest) -> Result<Vec<IbcEvent>, Error> {
-        self.send(|reply_to| ChainRequest::QueryPacketEventData { request, reply_to })
+        self.send(|reply_to| ChainRequest::QueryPacketEventDataFromTxs { request, reply_to })
+    }
+
+    fn query_blocks(
+        &self,
+        request: QueryBlockRequest,
+    ) -> Result<(Vec<IbcEvent>, Vec<IbcEvent>), Error> {
+        self.send(|reply_to| ChainRequest::QueryPacketEventDataFromBlocks { request, reply_to })
     }
 }
 
