@@ -1,30 +1,33 @@
-use std::{sync::Arc, thread};
+use alloc::sync::Arc;
+use std::thread;
 
 use crossbeam_channel as channel;
 use tokio::runtime::Runtime as TokioRuntime;
 use tracing::error;
 
 use ibc::{
+    core::{
+        ics02_client::{
+            client_consensus::{AnyConsensusState, AnyConsensusStateWithHeight, ConsensusState},
+            client_state::{AnyClientState, ClientState, IdentifiedAnyClientState},
+            events::UpdateClient,
+            header::{AnyHeader, Header},
+            misbehaviour::MisbehaviourEvidence,
+        },
+        ics03_connection::{
+            connection::{ConnectionEnd, IdentifiedConnectionEnd},
+            version::Version,
+        },
+        ics04_channel::{
+            channel::{ChannelEnd, IdentifiedChannelEnd},
+            packet::{PacketMsgType, Sequence},
+        },
+        ics23_commitment::commitment::CommitmentPrefix,
+        ics24_host::identifier::{ChannelId, ClientId, ConnectionId, PortId},
+    },
     events::IbcEvent,
-    ics02_client::{
-        client_consensus::{AnyConsensusState, AnyConsensusStateWithHeight, ConsensusState},
-        client_state::{AnyClientState, ClientState, IdentifiedAnyClientState},
-        events::UpdateClient,
-        header::{AnyHeader, Header},
-        misbehaviour::MisbehaviourEvidence,
-    },
-    ics03_connection::{
-        connection::{ConnectionEnd, IdentifiedConnectionEnd},
-        version::Version,
-    },
-    ics04_channel::{
-        channel::{ChannelEnd, IdentifiedChannelEnd},
-        packet::{PacketMsgType, Sequence},
-    },
-    ics23_commitment::commitment::CommitmentPrefix,
-    ics24_host::identifier::{ChannelId, ClientId, ConnectionId, PortId},
     proofs::Proofs,
-    query::QueryTxRequest,
+    query::{QueryBlockRequest, QueryTxRequest},
     signer::Signer,
     Height,
 };
@@ -41,6 +44,7 @@ use ibc_proto::ibc::core::{
 };
 
 use crate::{
+    chain::StatusResponse,
     config::ChainConfig,
     connection::ConnectionMsgType,
     error::Error,
@@ -54,7 +58,7 @@ use crate::{
 
 use super::{
     handle::{ChainHandle, ChainRequest, ReplyTo, Subscription},
-    ChainEndpoint,
+    ChainEndpoint, HealthCheck,
 };
 
 pub struct Threads {
@@ -195,6 +199,10 @@ where
                             break;
                         }
 
+                        Ok(ChainRequest::HealthCheck { reply_to }) => {
+                            self.health_check(reply_to)?
+                        },
+
                         Ok(ChainRequest::Subscribe { reply_to }) => {
                             self.subscribe(reply_to)?
                         },
@@ -211,6 +219,10 @@ where
                             self.get_signer(reply_to)?
                         }
 
+                        Ok(ChainRequest::Config { reply_to }) => {
+                            self.get_config(reply_to)?
+                        }
+
                         Ok(ChainRequest::Key { reply_to }) => {
                             self.get_key(reply_to)?
                         }
@@ -223,8 +235,8 @@ where
                             self.build_header(trusted_height, target_height, client_state, reply_to)?
                         }
 
-                        Ok(ChainRequest::BuildClientState { height, reply_to }) => {
-                            self.build_client_state(height, reply_to)?
+                        Ok(ChainRequest::BuildClientState { height, dst_config, reply_to }) => {
+                            self.build_client_state(height, dst_config, reply_to)?
                         }
 
                         Ok(ChainRequest::BuildConsensusState { trusted, target, client_state, reply_to }) => {
@@ -243,8 +255,8 @@ where
                             self.build_channel_proofs(port_id, channel_id, height, reply_to)?
                         },
 
-                        Ok(ChainRequest::QueryLatestHeight { reply_to }) => {
-                            self.query_latest_height(reply_to)?
+                        Ok(ChainRequest::QueryStatus { reply_to }) => {
+                            self.query_status(reply_to)?
                         }
 
                         Ok(ChainRequest::QueryClients { request, reply_to }) => {
@@ -343,10 +355,14 @@ where
                             self.query_next_sequence_receive(request, reply_to)?
                         },
 
-                        Ok(ChainRequest::QueryPacketEventData { request, reply_to }) => {
+                        Ok(ChainRequest::QueryPacketEventDataFromTxs { request, reply_to }) => {
                             self.query_txs(request, reply_to)?
                         },
 
+                        Ok(ChainRequest::QueryPacketEventDataFromBlocks { request, reply_to }) => {
+                            self.query_blocks(request, reply_to)?
+                        },
+
                         Err(e) => error!("received error via chain request channel: {}", e),
                     }
                 },
@@ -356,12 +372,14 @@ where
         Ok(())
     }
 
+    fn health_check(&mut self, reply_to: ReplyTo<HealthCheck>) -> Result<(), Error> {
+        let result = self.chain.health_check();
+        reply_to.send(result).map_err(Error::send)
+    }
+
     fn subscribe(&mut self, reply_to: ReplyTo<Subscription>) -> Result<(), Error> {
         let subscription = self.event_bus.subscribe();
-
-        reply_to.send(Ok(subscription)).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(Ok(subscription)).map_err(Error::send)
     }
 
     fn send_messages_and_wait_commit(
@@ -370,10 +388,7 @@ where
         reply_to: ReplyTo<Vec<IbcEvent>>,
     ) -> Result<(), Error> {
         let result = self.chain.send_messages_and_wait_commit(proto_msgs);
-
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
     fn send_messages_and_wait_check_tx(
@@ -382,42 +397,32 @@ where
         reply_to: ReplyTo<Vec<tendermint_rpc::endpoint::broadcast::tx_sync::Response>>,
     ) -> Result<(), Error> {
         let result = self.chain.send_messages_and_wait_check_tx(proto_msgs);
-
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
-    fn query_latest_height(&self, reply_to: ReplyTo<Height>) -> Result<(), Error> {
-        let latest_height = self.chain.query_latest_height();
-
-        reply_to.send(latest_height).map_err(Error::send)?;
-
-        Ok(())
+    fn query_status(&self, reply_to: ReplyTo<StatusResponse>) -> Result<(), Error> {
+        let latest_timestamp = self.chain.query_status();
+        reply_to.send(latest_timestamp).map_err(Error::send)
     }
 
     fn get_signer(&mut self, reply_to: ReplyTo<Signer>) -> Result<(), Error> {
         let result = self.chain.get_signer();
+        reply_to.send(result).map_err(Error::send)
+    }
 
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+    fn get_config(&self, reply_to: ReplyTo<ChainConfig>) -> Result<(), Error> {
+        let result = Ok(self.chain.config());
+        reply_to.send(result).map_err(Error::send)
     }
 
     fn get_key(&mut self, reply_to: ReplyTo<KeyEntry>) -> Result<(), Error> {
         let result = self.chain.get_key();
-
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
     fn module_version(&self, port_id: PortId, reply_to: ReplyTo<String>) -> Result<(), Error> {
         let result = self.chain.query_module_version(&port_id);
-
-        reply_to.send(Ok(result)).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(Ok(result)).map_err(Error::send)
     }
 
     fn build_header(
@@ -441,25 +446,22 @@ where
                 (header, support)
             });
 
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
     /// Constructs a client state for the given height
     fn build_client_state(
         &self,
         height: Height,
+        dst_config: ChainConfig,
         reply_to: ReplyTo<AnyClientState>,
     ) -> Result<(), Error> {
         let client_state = self
             .chain
-            .build_client_state(height)
+            .build_client_state(height, dst_config)
             .map(|cs| cs.wrap_any());
 
-        reply_to.send(client_state).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(client_state).map_err(Error::send)
     }
 
     /// Constructs a consensus state for the given height
@@ -477,9 +479,7 @@ where
             .build_consensus_state(verified.target)
             .map(|cs| cs.wrap_any());
 
-        reply_to.send(consensus_state).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(consensus_state).map_err(Error::send)
     }
 
     /// Constructs AnyMisbehaviour for the update event
@@ -493,9 +493,7 @@ where
             .light_client
             .check_misbehaviour(update_event, &client_state);
 
-        reply_to.send(misbehaviour).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(misbehaviour).map_err(Error::send)
     }
 
     fn build_connection_proofs_and_client_state(
@@ -516,9 +514,7 @@ where
         let result = result
             .map(|(opt_client_state, proofs)| (opt_client_state.map(|cs| cs.wrap_any()), proofs));
 
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
     fn query_clients(
@@ -527,10 +523,7 @@ where
         reply_to: ReplyTo<Vec<IdentifiedAnyClientState>>,
     ) -> Result<(), Error> {
         let result = self.chain.query_clients(request);
-
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
     fn query_client_connections(
@@ -539,10 +532,7 @@ where
         reply_to: ReplyTo<Vec<ConnectionId>>,
     ) -> Result<(), Error> {
         let result = self.chain.query_client_connections(request);
-
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
     fn query_client_state(
@@ -556,9 +546,7 @@ where
             .query_client_state(&client_id, height)
             .map(|cs| cs.wrap_any());
 
-        reply_to.send(client_state).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(client_state).map_err(Error::send)
     }
 
     fn query_upgraded_client_state(
@@ -571,9 +559,7 @@ where
             .query_upgraded_client_state(height)
             .map(|(cl, proof)| (cl.wrap_any(), proof));
 
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
     fn query_consensus_states(
@@ -582,10 +568,7 @@ where
         reply_to: ReplyTo<Vec<AnyConsensusStateWithHeight>>,
     ) -> Result<(), Error> {
         let consensus_states = self.chain.query_consensus_states(request);
-
-        reply_to.send(consensus_states).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(consensus_states).map_err(Error::send)
     }
 
     fn query_consensus_state(
@@ -599,9 +582,7 @@ where
             self.chain
                 .query_consensus_state(client_id, consensus_height, query_height);
 
-        reply_to.send(consensus_state).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(consensus_state).map_err(Error::send)
     }
 
     fn query_upgraded_consensus_state(
@@ -614,25 +595,17 @@ where
             .query_upgraded_consensus_state(height)
             .map(|(cs, proof)| (cs.wrap_any(), proof));
 
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
     fn query_commitment_prefix(&self, reply_to: ReplyTo<CommitmentPrefix>) -> Result<(), Error> {
         let prefix = self.chain.query_commitment_prefix();
-
-        reply_to.send(prefix).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(prefix).map_err(Error::send)
     }
 
     fn query_compatible_versions(&self, reply_to: ReplyTo<Vec<Version>>) -> Result<(), Error> {
         let versions = self.chain.query_compatible_versions();
-
-        reply_to.send(versions).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(versions).map_err(Error::send)
     }
 
     fn query_connection(
@@ -642,10 +615,7 @@ where
         reply_to: ReplyTo<ConnectionEnd>,
     ) -> Result<(), Error> {
         let connection_end = self.chain.query_connection(&connection_id, height);
-
-        reply_to.send(connection_end).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(connection_end).map_err(Error::send)
     }
 
     fn query_connections(
@@ -654,10 +624,7 @@ where
         reply_to: ReplyTo<Vec<IdentifiedConnectionEnd>>,
     ) -> Result<(), Error> {
         let result = self.chain.query_connections(request);
-
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
     fn query_connection_channels(
@@ -666,10 +633,7 @@ where
         reply_to: ReplyTo<Vec<IdentifiedChannelEnd>>,
     ) -> Result<(), Error> {
         let result = self.chain.query_connection_channels(request);
-
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
     fn query_channels(
@@ -678,10 +642,7 @@ where
         reply_to: ReplyTo<Vec<IdentifiedChannelEnd>>,
     ) -> Result<(), Error> {
         let result = self.chain.query_channels(request);
-
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
     fn query_channel(
@@ -692,10 +653,7 @@ where
         reply_to: ReplyTo<ChannelEnd>,
     ) -> Result<(), Error> {
         let result = self.chain.query_channel(&port_id, &channel_id, height);
-
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
     fn query_channel_client_state(
@@ -704,10 +662,7 @@ where
         reply_to: ReplyTo<Option<IdentifiedAnyClientState>>,
     ) -> Result<(), Error> {
         let result = self.chain.query_channel_client_state(request);
-
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
     fn proven_client_state(
@@ -721,9 +676,7 @@ where
             .proven_client_state(&client_id, height)
             .map(|(cs, mp)| (cs.wrap_any(), mp));
 
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
     fn proven_connection(
@@ -733,10 +686,7 @@ where
         reply_to: ReplyTo<(ConnectionEnd, MerkleProof)>,
     ) -> Result<(), Error> {
         let result = self.chain.proven_connection(&connection_id, height);
-
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
     fn proven_client_consensus(
@@ -751,9 +701,7 @@ where
             .proven_client_consensus(&client_id, consensus_height, height)
             .map(|(cs, mp)| (cs.wrap_any(), mp));
 
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
     fn build_channel_proofs(
@@ -767,9 +715,7 @@ where
             .chain
             .build_channel_proofs(&port_id, &channel_id, height);
 
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
     fn build_packet_proofs(
@@ -785,9 +731,7 @@ where
             self.chain
                 .build_packet_proofs(packet_type, port_id, channel_id, sequence, height);
 
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
     fn query_packet_commitments(
@@ -796,10 +740,7 @@ where
         reply_to: ReplyTo<(Vec<PacketState>, Height)>,
     ) -> Result<(), Error> {
         let result = self.chain.query_packet_commitments(request);
-
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
     fn query_unreceived_packets(
@@ -808,10 +749,7 @@ where
         reply_to: ReplyTo<Vec<u64>>,
     ) -> Result<(), Error> {
         let result = self.chain.query_unreceived_packets(request);
-
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
     fn query_packet_acknowledgements(
@@ -820,10 +758,7 @@ where
         reply_to: ReplyTo<(Vec<PacketState>, Height)>,
     ) -> Result<(), Error> {
         let result = self.chain.query_packet_acknowledgements(request);
-
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
     fn query_unreceived_acknowledgement(
@@ -832,10 +767,7 @@ where
         reply_to: ReplyTo<Vec<u64>>,
     ) -> Result<(), Error> {
         let result = self.chain.query_unreceived_acknowledgements(request);
-
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
     fn query_next_sequence_receive(
@@ -844,10 +776,7 @@ where
         reply_to: ReplyTo<Sequence>,
     ) -> Result<(), Error> {
         let result = self.chain.query_next_sequence_receive(request);
-
-        reply_to.send(result).map_err(Error::send)?;
-
-        Ok(())
+        reply_to.send(result).map_err(Error::send)
     }
 
     fn query_txs(
@@ -856,6 +785,15 @@ where
         reply_to: ReplyTo<Vec<IbcEvent>>,
     ) -> Result<(), Error> {
         let result = self.chain.query_txs(request);
+        reply_to.send(result).map_err(Error::send)
+    }
+
+    fn query_blocks(
+        &self,
+        request: QueryBlockRequest,
+        reply_to: ReplyTo<(Vec<IbcEvent>, Vec<IbcEvent>)>,
+    ) -> Result<(), Error> {
+        let result = self.chain.query_blocks(request);
 
         reply_to.send(result).map_err(Error::send)?;
 
