@@ -1,19 +1,21 @@
 #![allow(clippy::borrowed_box)]
 
+use core::time::Duration;
 use prost_types::Any;
 use serde::Serialize;
-use std::time::Duration;
 use tracing::{debug, error, info, warn};
 
+use ibc::core::ics04_channel::channel::{
+    ChannelEnd, Counterparty, IdentifiedChannelEnd, Order, State,
+};
+use ibc::core::ics04_channel::msgs::chan_close_confirm::MsgChannelCloseConfirm;
+use ibc::core::ics04_channel::msgs::chan_close_init::MsgChannelCloseInit;
+use ibc::core::ics04_channel::msgs::chan_open_ack::MsgChannelOpenAck;
+use ibc::core::ics04_channel::msgs::chan_open_confirm::MsgChannelOpenConfirm;
+use ibc::core::ics04_channel::msgs::chan_open_init::MsgChannelOpenInit;
+use ibc::core::ics04_channel::msgs::chan_open_try::MsgChannelOpenTry;
+use ibc::core::ics24_host::identifier::{ChainId, ChannelId, ClientId, ConnectionId, PortId};
 use ibc::events::IbcEvent;
-use ibc::ics04_channel::channel::{ChannelEnd, Counterparty, IdentifiedChannelEnd, Order, State};
-use ibc::ics04_channel::msgs::chan_close_confirm::MsgChannelCloseConfirm;
-use ibc::ics04_channel::msgs::chan_close_init::MsgChannelCloseInit;
-use ibc::ics04_channel::msgs::chan_open_ack::MsgChannelOpenAck;
-use ibc::ics04_channel::msgs::chan_open_confirm::MsgChannelOpenConfirm;
-use ibc::ics04_channel::msgs::chan_open_init::MsgChannelOpenInit;
-use ibc::ics04_channel::msgs::chan_open_try::MsgChannelOpenTry;
-use ibc::ics24_host::identifier::{ChainId, ChannelId, ClientId, ConnectionId, PortId};
 use ibc::tx_msg::Msg;
 use ibc::Height;
 use ibc_proto::ibc::core::channel::v1::QueryConnectionChannelsRequest;
@@ -31,7 +33,7 @@ pub mod error;
 pub use error::ChannelError;
 
 mod retry_strategy {
-    use std::time::Duration;
+    use core::time::Duration;
 
     use retry::delay::Fibonacci;
 
