@@ -3,16 +3,18 @@
 pub mod reload;
 pub mod types;
 
-use std::collections::{HashMap, HashSet};
-use std::{fmt, fs, fs::File, io::Write, path::Path, time::Duration};
+use alloc::collections::BTreeMap as HashMap;
+use alloc::collections::BTreeSet as HashSet;
+use core::{fmt, time::Duration};
+use std::{fs, fs::File, io::Write, path::Path};
 
 use serde_derive::{Deserialize, Serialize};
 use tendermint_light_client::types::TrustThreshold;
 
-use ibc::ics24_host::identifier::{ChainId, ChannelId, PortId};
+use ibc::core::ics24_host::identifier::{ChainId, ChannelId, PortId};
 use ibc::timestamp::ZERO_DURATION;
 
-use crate::config::types::{MaxMsgNum, MaxTxSize};
+use crate::config::types::{MaxMsgNum, MaxTxSize, Memo};
 use crate::error::Error;
 
 #[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
@@ -79,6 +81,10 @@ impl ChannelsSpec {
 pub mod default {
     use super::*;
 
+    pub fn tx_confirmation() -> bool {
+        true
+    }
+
     pub fn filter() -> bool {
         false
     }
@@ -91,14 +97,14 @@ pub mod default {
         Duration::from_secs(10)
     }
 
-    pub fn trusting_period() -> Duration {
-        Duration::from_secs(336 * 60 * 60) // 336 hours ~ 14 days
-    }
-
     pub fn clock_drift() -> Duration {
         Duration::from_secs(5)
     }
 
+    pub fn max_block_time() -> Duration {
+        Duration::from_secs(10)
+    }
+
     pub fn connection_delay() -> Duration {
         ZERO_DURATION
     }
@@ -213,6 +219,8 @@ pub struct GlobalConfig {
     pub filter: bool,
     #[serde(default = "default::clear_packets_interval")]
     pub clear_packets_interval: u64,
+    #[serde(default = "default::tx_confirmation")]
+    pub tx_confirmation: bool,
 }
 
 impl Default for GlobalConfig {
@@ -222,6 +230,7 @@ impl Default for GlobalConfig {
             log_level: LogLevel::default(),
             filter: default::filter(),
             clear_packets_interval: default::clear_packets_interval(),
+            tx_confirmation: default::tx_confirmation(),
         }
     }
 }
@@ -262,6 +271,37 @@ impl Default for RestConfig {
     }
 }
 
+/// It defines the address generation method
+/// TODO: Ethermint `pk_type` to be restricted
+/// after the Cosmos SDK release with ethsecp256k1
+/// https://github.com/cosmos/cosmos-sdk/pull/9981
+#[derive(Clone, Debug, PartialEq, Deserialize, Serialize)]
+#[serde(
+    rename_all = "lowercase",
+    tag = "derivation",
+    content = "proto_type",
+    deny_unknown_fields
+)]
+pub enum AddressType {
+    Cosmos,
+    Ethermint { pk_type: String },
+}
+
+impl Default for AddressType {
+    fn default() -> Self {
+        AddressType::Cosmos
+    }
+}
+
+impl fmt::Display for AddressType {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            AddressType::Cosmos => write!(f, "cosmos"),
+            AddressType::Ethermint { .. } => write!(f, "ethermint"),
+        }
+    }
+}
+
 #[derive(Clone, Debug, Deserialize, Serialize)]
 #[serde(deny_unknown_fields)]
 pub struct ChainConfig {
@@ -274,6 +314,7 @@ pub struct ChainConfig {
     pub account_prefix: String,
     pub key_name: String,
     pub store_prefix: String,
+    pub default_gas: Option<u64>,
     pub max_gas: Option<u64>,
     pub gas_adjustment: Option<f64>,
     #[serde(default)]
@@ -282,8 +323,12 @@ pub struct ChainConfig {
     pub max_tx_size: MaxTxSize,
     #[serde(default = "default::clock_drift", with = "humantime_serde")]
     pub clock_drift: Duration,
-    #[serde(default = "default::trusting_period", with = "humantime_serde")]
-    pub trusting_period: Duration,
+    #[serde(default = "default::max_block_time", with = "humantime_serde")]
+    pub max_block_time: Duration,
+    #[serde(default, with = "humantime_serde")]
+    pub trusting_period: Option<Duration>,
+    #[serde(default)]
+    pub memo_prefix: Memo,
 
     // these two need to be last otherwise we run into `ValueAfterTable` error when serializing to TOML
     #[serde(default)]
@@ -291,6 +336,8 @@ pub struct ChainConfig {
     pub gas_price: GasPrice,
     #[serde(default)]
     pub packet_filter: PacketFilter,
+    #[serde(default)]
+    pub address_type: AddressType,
 }
 
 /// Attempt to load and parse the TOML config file as a `Config`.
