@@ -1,15 +1,13 @@
 //! Facility for reloading the relayer configuration.
 
-use std::{
-    path::PathBuf,
-    sync::{Arc, RwLock},
-};
+use alloc::sync::Arc;
+use std::{path::PathBuf, sync::RwLock};
 
 use crossbeam_channel::Sender;
 use itertools::Itertools;
 use thiserror::Error;
 
-use ibc::ics24_host::identifier::ChainId;
+use ibc::core::ics24_host::identifier::ChainId;
 use tracing::debug;
 
 use crate::{
