@@ -1,4 +1,4 @@
-use std::time::Duration;
+use core::time::Duration;
 
 use crate::chain::counterparty::connection_state_on_destination;
 use crate::util::retry::RetryResult;
@@ -9,16 +9,16 @@ use serde::Serialize;
 use tracing::debug;
 use tracing::{error, warn};
 
-use ibc::events::IbcEvent;
-use ibc::ics02_client::height::Height;
-use ibc::ics03_connection::connection::{
+use ibc::core::ics02_client::height::Height;
+use ibc::core::ics03_connection::connection::{
     ConnectionEnd, Counterparty, IdentifiedConnectionEnd, State,
 };
-use ibc::ics03_connection::msgs::conn_open_ack::MsgConnectionOpenAck;
-use ibc::ics03_connection::msgs::conn_open_confirm::MsgConnectionOpenConfirm;
-use ibc::ics03_connection::msgs::conn_open_init::MsgConnectionOpenInit;
-use ibc::ics03_connection::msgs::conn_open_try::MsgConnectionOpenTry;
-use ibc::ics24_host::identifier::{ChainId, ClientId, ConnectionId};
+use ibc::core::ics03_connection::msgs::conn_open_ack::MsgConnectionOpenAck;
+use ibc::core::ics03_connection::msgs::conn_open_confirm::MsgConnectionOpenConfirm;
+use ibc::core::ics03_connection::msgs::conn_open_init::MsgConnectionOpenInit;
+use ibc::core::ics03_connection::msgs::conn_open_try::MsgConnectionOpenTry;
+use ibc::core::ics24_host::identifier::{ChainId, ClientId, ConnectionId};
+use ibc::events::IbcEvent;
 use ibc::timestamp::ZERO_DURATION;
 use ibc::tx_msg::Msg;
 
@@ -1122,6 +1122,7 @@ fn check_destination_connection_state(
             == expected_connection.counterparty().connection_id();
 
     // TODO check versions and store prefix
+    // https://github.com/informalsystems/ibc-rs/issues/1389
 
     if good_state && good_client_ids && good_connection_ids {
         Ok(())
