@@ -1,34 +1,40 @@
 //! This module defines the various errors that be raised in the relayer.
 
-use crate::keyring::errors::Error as KeyringError;
-use crate::sdk_error::SdkError;
+use core::time::Duration;
+
 use flex_error::{define_error, DisplayOnly, TraceClone, TraceError};
 use http::uri::InvalidUri;
+use humantime::format_duration;
 use prost::DecodeError;
+use tendermint::Error as TendermintError;
 use tendermint_light_client::{
     components::io::IoError as LightClientIoError, errors::Error as LightClientError,
 };
 use tendermint_proto::Error as TendermintProtoError;
 use tendermint_rpc::endpoint::abci_query::AbciQuery;
 use tendermint_rpc::endpoint::broadcast::tx_commit::TxResult;
-use tendermint_rpc::Error as TendermintError;
+use tendermint_rpc::Error as TendermintRpcError;
 use tonic::{
     metadata::errors::InvalidMetadataValue, transport::Error as TransportError,
     Status as GrpcStatus,
 };
 
 use ibc::{
-    ics02_client::{client_type::ClientType, error as client_error},
-    ics03_connection::error as connection_error,
-    ics07_tendermint::error as tendermint_error,
-    ics18_relayer::error as relayer_error,
-    ics23_commitment::error as commitment_error,
-    ics24_host::identifier::{ChainId, ChannelId, ConnectionId},
+    clients::ics07_tendermint::error as tendermint_error,
+    core::{
+        ics02_client::{client_type::ClientType, error as client_error},
+        ics03_connection::error as connection_error,
+        ics23_commitment::error as commitment_error,
+        ics24_host::identifier::{ChainId, ChannelId, ConnectionId},
+    },
     proofs::ProofError,
+    relayer::ics18_relayer::error as relayer_error,
 };
 
 use crate::chain::cosmos::GENESIS_MAX_BYTES_MAX_FRACTION;
 use crate::event::monitor;
+use crate::keyring::errors::Error as KeyringError;
+use crate::sdk_error::SdkError;
 
 define_error! {
     Error {
@@ -50,7 +56,7 @@ define_error! {
 
         Rpc
             { url: tendermint_rpc::Url }
-            [ TraceClone<TendermintError> ]
+            [ TraceClone<TendermintRpcError> ]
             |e| { format!("RPC error to endpoint {}", e.url) },
 
         AbciQuery
@@ -120,14 +126,10 @@ define_error! {
             |e| { format!("node at {} running chain {} not caught up", e.address, e.chain_id) },
 
         PrivateStore
-            |_| { "requested proof for a path in the private store" },
-
-        Store
-            [ TraceError<sled::Error> ]
-            |_| { "Store error" },
+            |_| { "Requested proof for a path in the private store" },
 
         Event
-            |_| { "Bad Notification" },
+            |_| { "Bad notification" },
 
         ConversionFromAny
             [ TraceError<TendermintProtoError> ]
@@ -154,7 +156,7 @@ define_error! {
             |_| { "Malformed proof" },
 
         InvalidHeight
-            [ DisplayOnly<Box<dyn std::error::Error + Send + Sync>> ]
+            [ TendermintError ]
             |_| { "Invalid height" },
 
         InvalidMetadata
@@ -287,7 +289,7 @@ define_error! {
             |_| { "requested proof for data in the privateStore" },
 
         ChannelSend
-            |_| { "failed to send through channel" },
+            |_| { "internal communication failure: error sending inter-thread request/response" },
 
         ChannelReceive
             [ TraceError<crossbeam_channel::RecvError> ]
@@ -305,7 +307,7 @@ define_error! {
 
         InvalidKeyAddress
             { address: String }
-            [ DisplayOnly<Box<dyn std::error::Error + Send + Sync>> ]
+            [ TendermintError ]
             |e| { format!("invalid key address: {0}", e.address) },
 
         Bech32Encoding
@@ -412,6 +414,38 @@ define_error! {
                     e.chain_id, e.configured_bound, GENESIS_MAX_BYTES_MAX_FRACTION * 100.0, e.genesis_bound)
             },
 
+        ConfigValidationTrustingPeriodSmallerThanZero
+            {
+                chain_id: ChainId,
+                trusting_period: Duration,
+            }
+            |e| {
+                format!("semantic config validation failed for option `trusting_period` of chain '{}', reason: trusting period ({}) must be greater than zero",
+                    e.chain_id, format_duration(e.trusting_period))
+            },
+
+        ConfigValidationTrustingPeriodGreaterThanUnbondingPeriod
+            {
+                chain_id: ChainId,
+                trusting_period: Duration,
+                unbonding_period: Duration,
+            }
+            |e| {
+                format!("semantic config validation failed for option `trusting_period` of chain '{}', reason: trusting period ({}) must be smaller than the unbonding period ({})",
+                    e.chain_id, format_duration(e.trusting_period), format_duration(e.unbonding_period))
+            },
+
+        ConfigValidationDefaultGasTooHigh
+            {
+                chain_id: ChainId,
+                default_gas: u64,
+                max_gas: u64,
+            }
+            |e| {
+                format!("semantic config validation failed for option `default_gas` of chain '{}', reason: default gas ({}) must be smaller than the max gas ({})",
+                    e.chain_id, e.default_gas, e.max_gas)
+            },
+
         SdkModuleVersion
             {
                 chain_id: ChainId,
@@ -422,6 +456,19 @@ define_error! {
                 format!("Hermes health check failed while verifying the application compatibility for chain {0}:{1}; caused by: {2}",
                     e.chain_id, e.address, e.cause)
             },
+
+        UnknownAccountType
+            {
+                type_url: String
+            }
+            |e| {
+                format!("Failed to deserialize account of an unknown protobuf type: {0}",
+                    e.type_url)
+            },
+
+        EmptyBaseAccount
+            |_| { "Empty BaseAccount within EthAccount" },
+
     }
 }
 
