@@ -1,25 +1,30 @@
-use std::cell::RefCell;
-use std::collections::{HashMap, VecDeque};
+use alloc::collections::BTreeMap as HashMap;
+use alloc::collections::VecDeque;
+use core::cell::RefCell;
+use core::fmt;
+use std::thread;
 use std::time::Instant;
-use std::{fmt, thread};
 
 use itertools::Itertools;
 use prost_types::Any;
 use tracing::{debug, error, info, trace};
 
 use ibc::{
-    events::{IbcEvent, IbcEventType, PrettyEvents},
-    ics04_channel::{
-        channel::{ChannelEnd, Order, QueryPacketEventDataRequest, State as ChannelState},
-        events::{SendPacket, WriteAcknowledgement},
-        msgs::{
-            acknowledgement::MsgAcknowledgement, chan_close_confirm::MsgChannelCloseConfirm,
-            recv_packet::MsgRecvPacket, timeout::MsgTimeout, timeout_on_close::MsgTimeoutOnClose,
+    core::{
+        ics04_channel::{
+            channel::{ChannelEnd, Order, QueryPacketEventDataRequest, State as ChannelState},
+            events::{SendPacket, WriteAcknowledgement},
+            msgs::{
+                acknowledgement::MsgAcknowledgement, chan_close_confirm::MsgChannelCloseConfirm,
+                recv_packet::MsgRecvPacket, timeout::MsgTimeout,
+                timeout_on_close::MsgTimeoutOnClose,
+            },
+            packet::{Packet, PacketMsgType, Sequence},
         },
-        packet::{Packet, PacketMsgType, Sequence},
+        ics24_host::identifier::{ChannelId, ClientId, ConnectionId, PortId},
     },
-    ics24_host::identifier::{ChannelId, ClientId, ConnectionId, PortId},
-    query::QueryTxRequest,
+    events::{IbcEvent, PrettyEvents, WithBlockDataType},
+    query::{QueryBlockRequest, QueryTxRequest},
     signer::Signer,
     timestamp::ZERO_DURATION,
     tx_msg::Msg,
@@ -33,6 +38,7 @@ use crate::chain::counterparty::{
     unreceived_acknowledgements_sequences, unreceived_packets_sequences,
 };
 use crate::chain::handle::ChainHandle;
+use crate::chain::StatusResponse;
 use crate::channel::error::ChannelError;
 use crate::channel::Channel;
 use crate::event::monitor::EventBatch;
@@ -71,15 +77,21 @@ pub struct RelayPath<ChainA: ChainHandle, ChainB: ChainHandle> {
     src_operational_data: Queue<OperationalData>,
     dst_operational_data: Queue<OperationalData>,
 
-    // The mediator stores pending operational data.
-    // The relaying path periodically invokes the mediator
-    // to confirm transactions.
+    // Toggle for the transaction confirmation mechanism.
+    confirm_txes: bool,
+
+    // Stores pending (i.e., unconfirmed) operational data.
+    // The relaying path periodically tries to confirm these pending
+    // transactions if [`confirm_txes`] is true.
     pending_txs_src: PendingTxs<ChainA>,
     pending_txs_dst: PendingTxs<ChainB>,
 }
 
 impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
-    pub fn new(channel: Channel<ChainA, ChainB>) -> Result<Self, LinkError> {
+    pub fn new(
+        channel: Channel<ChainA, ChainB>,
+        with_tx_confirmation: bool,
+    ) -> Result<Self, LinkError> {
         let src_chain = channel.src_chain().clone();
         let dst_chain = channel.dst_chain().clone();
 
@@ -110,6 +122,8 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
             clear_packets: RefCell::new(true),
             src_operational_data: Queue::new(),
             dst_operational_data: Queue::new(),
+
+            confirm_txes: with_tx_confirmation,
             pending_txs_src: PendingTxs::new(src_chain, src_channel_id, src_port_id, dst_chain_id),
             pending_txs_dst: PendingTxs::new(dst_chain, dst_channel_id, dst_port_id, src_chain_id),
         })
@@ -365,9 +379,13 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
             Some(ev) => ev.height(),
         };
 
-        let dst_height = self.dst_latest_height()?;
+        let dst_latest_info = self
+            .dst_chain()
+            .query_status()
+            .map_err(|e| LinkError::query(self.src_chain().id(), e))?;
+        let dst_latest_height = dst_latest_info.height;
         // Operational data targeting the source chain (e.g., Timeout packets)
-        let mut src_od = OperationalData::new(dst_height, OperationalDataTarget::Source);
+        let mut src_od = OperationalData::new(dst_latest_height, OperationalDataTarget::Source);
         // Operational data targeting the destination chain (e.g., SendPacket messages)
         let mut dst_od = OperationalData::new(src_height, OperationalDataTarget::Destination);
 
@@ -404,7 +422,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
                     } else {
                         self.build_recv_or_timeout_from_send_packet_event(
                             send_packet_ev,
-                            dst_height,
+                            &dst_latest_info,
                         )?
                     }
                 }
@@ -632,6 +650,10 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
     }
 
     fn enqueue_pending_tx(&self, reply: AsyncReply, odata: OperationalData) {
+        if !self.confirm_txes {
+            return;
+        }
+
         match odata.target {
             OperationalDataTarget::Source => {
                 self.pending_txs_src.insert_new_pending_tx(reply, odata);
@@ -820,7 +842,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
         }
 
         debug!(
-            "[{}] packets that still have commitments on {}: {} (first 10 shown here; total={})",
+            "[{}] packet seq. that still have commitments on {}: {} (first 10 shown here; total={})",
             self,
             self.src_chain().id(),
             commit_sequences.iter().take(10).join(", "),
@@ -835,40 +857,75 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
             sequences.iter().take(10).join(", "), sequences.len()
         );
 
-        let query = QueryTxRequest::Packet(QueryPacketEventDataRequest {
-            event_id: IbcEventType::SendPacket,
+        let mut query = QueryPacketEventDataRequest {
+            event_id: WithBlockDataType::SendPacket,
             source_port_id: self.src_port_id().clone(),
             source_channel_id: src_channel_id.clone(),
             destination_port_id: self.dst_port_id().clone(),
             destination_channel_id: dst_channel_id.clone(),
             sequences,
             height: query_height,
-        });
+        };
 
-        events_result = self
+        let tx_events = self
             .src_chain()
-            .query_txs(query)
+            .query_txs(QueryTxRequest::Packet(query.clone()))
             .map_err(LinkError::relayer)?;
 
-        let mut packet_sequences = vec![];
-        for event in events_result.iter() {
-            match event {
-                IbcEvent::SendPacket(send_event) => {
-                    packet_sequences.push(send_event.packet.sequence);
-                    if packet_sequences.len() > 10 {
-                        // Enough to print the first 10
-                        break;
+        let recvd_sequences: Vec<Sequence> = tx_events
+            .iter()
+            .filter_map(|ev| match ev {
+                IbcEvent::SendPacket(ref send_ev) => Some(send_ev.packet.sequence),
+                IbcEvent::WriteAcknowledgement(ref ack_ev) => Some(ack_ev.packet.sequence),
+                _ => None,
+            })
+            .collect();
+        query.sequences.retain(|seq| !recvd_sequences.contains(seq));
+
+        let (start_block_events, end_block_events) = if !query.sequences.is_empty() {
+            self.src_chain()
+                .query_blocks(QueryBlockRequest::Packet(query))
+                .map_err(LinkError::relayer)?
+        } else {
+            Default::default()
+        };
+
+        trace!("[{}] start_block_events {:?}", self, start_block_events);
+        trace!("[{}] tx_events {:?}", self, tx_events);
+        trace!("[{}] end_block_events {:?}", self, end_block_events);
+
+        // events must be ordered in the following fashion -
+        // start-block events followed by tx-events followed by end-block events
+        events_result.extend(start_block_events);
+        events_result.extend(tx_events);
+        events_result.extend(end_block_events);
+
+        if events_result.is_empty() {
+            info!(
+                "[{}] found zero unprocessed SendPacket events on source chain, nothing to do",
+                self
+            );
+        } else {
+            let mut packet_sequences = vec![];
+            for event in events_result.iter() {
+                match event {
+                    IbcEvent::SendPacket(send_event) => {
+                        packet_sequences.push(send_event.packet.sequence);
+                        if packet_sequences.len() >= 10 {
+                            // Enough to print the first 10
+                            break;
+                        }
                     }
+                    _ => return Err(LinkError::unexpected_event(event.clone())),
                 }
-                _ => return Err(LinkError::unexpected_event(event.clone())),
             }
+            info!(
+                "[{}] found unprocessed SendPacket events for {:?} (first 10 shown here; total={})",
+                self,
+                packet_sequences,
+                events_result.len()
+            );
         }
-        info!(
-            "[{}] found unprocessed SendPacket events for {:?} (first 10 shown here; total={})",
-            self,
-            packet_sequences,
-            events_result.len()
-        );
 
         Ok((events_result, query_height))
     }
@@ -922,7 +979,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
         events_result = self
             .src_chain()
             .query_txs(QueryTxRequest::Packet(QueryPacketEventDataRequest {
-                event_id: IbcEventType::WriteAck,
+                event_id: WithBlockDataType::WriteAck,
                 source_port_id: self.dst_port_id().clone(),
                 source_channel_id: dst_channel_id.clone(),
                 destination_port_id: self.src_port_id().clone(),
@@ -932,22 +989,29 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
             }))
             .map_err(|e| LinkError::query(self.src_chain().id(), e))?;
 
-        let mut packet_sequences = vec![];
-        for event in events_result.iter() {
-            match event {
-                IbcEvent::WriteAcknowledgement(write_ack_event) => {
-                    packet_sequences.push(write_ack_event.packet.sequence);
-                    if packet_sequences.len() > 10 {
-                        // Enough to print the first 10
-                        break;
+        if events_result.is_empty() {
+            info!(
+                "[{}] found zero unprocessed WriteAcknowledgement events on source chain, nothing to do",
+                self
+            );
+        } else {
+            let mut packet_sequences = vec![];
+            for event in events_result.iter() {
+                match event {
+                    IbcEvent::WriteAcknowledgement(write_ack_event) => {
+                        packet_sequences.push(write_ack_event.packet.sequence);
+                        if packet_sequences.len() >= 10 {
+                            // Enough to print the first 10
+                            break;
+                        }
+                    }
+                    _ => {
+                        return Err(LinkError::unexpected_event(event.clone()));
                     }
                 }
-                _ => {
-                    return Err(LinkError::unexpected_event(event.clone()));
-                }
             }
+            info!("[{}] found unprocessed WriteAcknowledgement events for {:?} (first 10 shown here; total={})", self, packet_sequences, events_result.len());
         }
-        info!("[{}] found unprocessed WriteAcknowledgement events for {:?} (first 10 shown here; total={})", self, packet_sequences, events_result.len());
 
         Ok((events_result, query_height))
     }
@@ -1142,16 +1206,16 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
     fn build_timeout_from_send_packet_event(
         &self,
         event: &SendPacket,
-        dst_chain_height: Height,
+        dst_info: &StatusResponse,
     ) -> Result<Option<Any>, LinkError> {
         let packet = event.packet.clone();
         if self
-            .dst_channel(dst_chain_height)?
+            .dst_channel(dst_info.height)?
             .state_matches(&ChannelState::Closed)
         {
-            Ok(self.build_timeout_on_close_packet(&event.packet, dst_chain_height)?)
-        } else if packet.timed_out(dst_chain_height) {
-            Ok(self.build_timeout_packet(&event.packet, dst_chain_height)?)
+            Ok(self.build_timeout_on_close_packet(&event.packet, dst_info.height)?)
+        } else if packet.timed_out(&dst_info.timestamp, dst_info.height) {
+            Ok(self.build_timeout_packet(&event.packet, dst_info.height)?)
         } else {
             Ok(None)
         }
@@ -1160,9 +1224,9 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
     fn build_recv_or_timeout_from_send_packet_event(
         &self,
         event: &SendPacket,
-        dst_chain_height: Height,
+        dst_info: &StatusResponse,
     ) -> Result<(Option<Any>, Option<Any>), LinkError> {
-        let timeout = self.build_timeout_from_send_packet_event(event, dst_chain_height)?;
+        let timeout = self.build_timeout_from_send_packet_event(event, dst_info)?;
         if timeout.is_some() {
             Ok((None, timeout))
         } else {
@@ -1197,6 +1261,10 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
     }
 
     pub fn process_pending_txs(&self) -> RelaySummary {
+        if !self.confirm_txes {
+            return RelaySummary::empty();
+        }
+
         let mut summary_src = self.process_pending_txs_src().unwrap_or_else(|e| {
             error!("error processing pending events in source chain: {}", e);
             RelaySummary::empty()
@@ -1240,7 +1308,17 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
     /// Verifies if any sendPacket messages timed-out. If so, moves them from destination op. data
     /// to source operational data, and adjusts the events and messages accordingly.
     pub fn refresh_schedule(&self) -> Result<(), LinkError> {
-        let dst_current_height = self.dst_latest_height()?;
+        // Bail fast if no op. data to refresh
+        if self.dst_operational_data.is_empty() {
+            return Ok(());
+        }
+
+        let dst_status = self
+            .dst_chain()
+            .query_status()
+            .map_err(|e| LinkError::query(self.src_chain().id(), e))?;
+
+        let dst_current_height = dst_status.height;
 
         // Intermediary data struct to help better manage the transfer from dst. operational data
         // to source operational data.
@@ -1265,7 +1343,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
                                 self, e
                             );
                         } else if let Some(new_msg) =
-                            self.build_timeout_from_send_packet_event(e, dst_current_height)?
+                            self.build_timeout_from_send_packet_event(e, &dst_status)?
                         {
                             debug!(
                                 "[{}] refreshing schedule: found a timed-out msg in the op data {}",
