@@ -1,15 +1,14 @@
 //! Registry for keeping track of [`ChainHandle`]s indexed by a `ChainId`.
 
-use std::{
-    collections::HashMap,
-    sync::{Arc, RwLock},
-};
+use alloc::collections::btree_map::BTreeMap as HashMap;
+use alloc::sync::Arc;
+use std::sync::RwLock;
 
 use flex_error::define_error;
 use tokio::runtime::Runtime as TokioRuntime;
 use tracing::{trace, warn};
 
-use ibc::ics24_host::identifier::ChainId;
+use ibc::core::ics24_host::identifier::ChainId;
 
 use crate::{
     chain::{handle::ChainHandle, runtime::ChainRuntime, CosmosSdkChain},
@@ -97,7 +96,7 @@ impl<Chain: ChainHandle> Registry<Chain> {
         if !self.handles.contains_key(chain_id) {
             let handle = spawn_chain_runtime(&self.config, chain_id, self.rt.clone())?;
             self.handles.insert(chain_id.clone(), handle);
-            trace!("spawned chain runtime for chain {}", chain_id);
+            trace!("[{}] spawned chain runtime", chain_id);
             Ok(true)
         } else {
             Ok(false)
