@@ -1,16 +1,16 @@
-use std::{thread, time::Duration};
+use core::time::Duration;
+use std::{thread, time::Instant};
 
 use crossbeam_channel::Receiver;
 use tracing::{debug, info, trace, warn};
 
-use ibc::{events::IbcEvent, ics02_client::events::UpdateClient};
+use ibc::{core::ics02_client::events::UpdateClient, events::IbcEvent};
 
 use crate::{
     chain::handle::{ChainHandle, ChainHandlePair},
     foreign_client::{ForeignClient, ForeignClientErrorDetail, MisbehaviourResults},
     object::Client,
     telemetry,
-    telemetry::Telemetry,
 };
 
 use super::error::RunError;
@@ -20,9 +20,6 @@ pub struct ClientWorker<ChainA: ChainHandle, ChainB: ChainHandle> {
     client: Client,
     chains: ChainHandlePair<ChainA, ChainB>,
     cmd_rx: Receiver<WorkerCmd>,
-
-    #[allow(dead_code)]
-    telemetry: Telemetry,
 }
 
 impl<ChainA: ChainHandle, ChainB: ChainHandle> ClientWorker<ChainA, ChainB> {
@@ -30,13 +27,11 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> ClientWorker<ChainA, ChainB> {
         client: Client,
         chains: ChainHandlePair<ChainA, ChainB>,
         cmd_rx: Receiver<WorkerCmd>,
-        telemetry: Telemetry,
     ) -> Self {
         Self {
             client,
             chains,
             cmd_rx,
-            telemetry,
         }
     }
 
@@ -56,31 +51,40 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> ClientWorker<ChainA, ChainB> {
         // initial check for evidence of misbehaviour for all updates
         let skip_misbehaviour = self.detect_misbehaviour(&client, None);
 
+        // remember the time of the last refresh so we backoff
+        let mut last_refresh = Instant::now() - Duration::from_secs(61);
+
         loop {
             thread::sleep(Duration::from_millis(600));
 
-            // Run client refresh, exit only if expired or frozen
-            match client.refresh() {
-                Ok(Some(_)) => {
-                    telemetry! {
-                        self.telemetry.ibc_client_update(
+            // Clients typically need refresh every 2/3 of their
+            // trusting period (which can e.g., two weeks).
+            // Backoff refresh checking to attempt it every minute.
+            if last_refresh.elapsed() > Duration::from_secs(60) {
+                // Run client refresh, exit only if expired or frozen
+                match client.refresh() {
+                    Ok(Some(_)) => {
+                        telemetry!(
+                            ibc_client_update,
                             &self.client.dst_chain_id,
                             &self.client.dst_client_id,
                             1
-                        )
-                    };
-                }
-                Err(e) => {
-                    if let ForeignClientErrorDetail::ExpiredOrFrozen(_) = e.detail() {
-                        warn!("failed to refresh client '{}': {}", client, e);
-
-                        // This worker has completed its job as the client cannot be refreshed any
-                        // further, and can therefore exit without an error.
-                        return Ok(());
+                        );
                     }
-                }
-                _ => (),
-            };
+                    Err(e) => {
+                        if let ForeignClientErrorDetail::ExpiredOrFrozen(_) = e.detail() {
+                            warn!("failed to refresh client '{}': {}", client, e);
+
+                            // This worker has completed its job as the client cannot be refreshed any
+                            // further, and can therefore exit without an error.
+                            return Ok(());
+                        }
+                    }
+                    _ => (),
+                };
+
+                last_refresh = Instant::now();
+            }
 
             if skip_misbehaviour {
                 continue;
@@ -109,13 +113,12 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> ClientWorker<ChainA, ChainB> {
                         // Run misbehaviour. If evidence submitted the loop will exit in next
                         // iteration with frozen client
                         if self.detect_misbehaviour(client, Some(update)) {
-                            telemetry! {
-                                self.telemetry.ibc_client_misbehaviour(
-                                    &self.client.dst_chain_id,
-                                    &self.client.dst_client_id,
-                                    1
-                                )
-                            };
+                            telemetry!(
+                                ibc_client_misbehaviour,
+                                &self.client.dst_chain_id,
+                                &self.client.dst_client_id,
+                                1
+                            );
                         }
                     }
                 }
