@@ -1,4 +1,4 @@
-use ibc::{ics02_client::events::NewBlock, Height};
+use ibc::{core::ics02_client::events::NewBlock, Height};
 
 use crate::event::monitor::EventBatch;
 
@@ -8,9 +8,12 @@ pub enum WorkerCmd {
     /// A batch of packet events need to be relayed
     IbcEvents { batch: EventBatch },
 
-    /// A batch of [`NewBlock`] events need to be relayed
+    /// A new block has been committed
     NewBlock { height: Height, new_block: NewBlock },
 
+    /// Trigger a pending packets clear
+    ClearPendingPackets,
+
     /// Shutdown the worker
     Shutdown,
 }
