@@ -1,13 +1,13 @@
-use std::{
-    fmt,
-    thread::{self, JoinHandle},
-};
+use core::fmt;
+use std::thread::{self, JoinHandle};
 
 use crossbeam_channel::Sender;
 use tracing::trace;
 
 use ibc::{
-    events::IbcEvent, ics02_client::events::NewBlock, ics24_host::identifier::ChainId, Height,
+    core::{ics02_client::events::NewBlock, ics24_host::identifier::ChainId},
+    events::IbcEvent,
+    Height,
 };
 
 use crate::{event::monitor::EventBatch, object::Object};
